#include "CalcScores.hpp"
#include "Game.hpp"
#include "Map.hpp"

using std::vector;

void calcScores(const Game& game,
                vector<int64_t>* pScores)
{
    auto& scores = *pScores;
    scores.resize(game.m_numPunters);

    vector<uint32_t> queueCurrent;
    vector<uint32_t> queueNext;
    vector<uint32_t> dists(game.m_map.m_numSites);
    vector<uint8_t> visited(game.m_map.m_numSites);

    for (uint32_t iMine = 0; iMine < game.m_map.m_numMines; iMine++)
    {
        for (uint32_t iSite = 0; iSite < game.m_map.m_numSites; iSite++)
        {
            dists[iSite] = (uint32_t)-1;
        }

        uint32_t dist = 0;
        queueNext.push_back(game.m_map.m_mines[iMine]);
        while (!queueNext.empty())
        {
            queueCurrent.swap(queueNext);
            for (uint32_t iCurSite : queueCurrent)
            {
                if (dists[iCurSite] != (uint32_t)-1)
                {
                    continue;
                }

                dists[iCurSite] = dist;

                uint32_t low = game.m_map.m_siteRiverBounds[iCurSite].m_low;
                uint32_t high = game.m_map.m_siteRiverBounds[iCurSite].m_high;
                for (uint32_t iSiteRiver = low; iSiteRiver < high; iSiteRiver++)
                {
                    auto& siteRiver = game.m_map.m_siteRivers[iSiteRiver];
                    uint32_t target = siteRiver.m_target;
                    if (dists[target] == (uint32_t)-1)
                    {
                        queueNext.push_back(target);
                    }
                }
            }
            queueCurrent.clear();
            dist++;
        }

        for (uint32_t iPunter = 0; iPunter < game.m_numPunters; iPunter++)
        {
            for (uint32_t iSite = 0; iSite < game.m_map.m_numSites; iSite++)
            {
                visited[iSite] = 0;
            }

            queueNext.push_back(game.m_map.m_mines[iMine]);
            while (!queueNext.empty())
            {
                queueCurrent.swap(queueNext);
                for (uint32_t iCurSite : queueCurrent)
                {
                    if (visited[iCurSite])
                    {
                        continue;
                    }

                    visited[iCurSite] = 1;
                    uint32_t curDist = dists[iCurSite];
                    scores[iPunter] += curDist * curDist;

                    uint32_t low = game.m_map.m_siteRiverBounds[iCurSite].m_low;
                    uint32_t high = game.m_map.m_siteRiverBounds[iCurSite].m_high;
                    for (uint32_t iSiteRiver = low; iSiteRiver < high; iSiteRiver++)
                    {
                        auto& siteRiver = game.m_map.m_siteRivers[iSiteRiver];
                        if (!(game.m_claims[siteRiver.m_river] == iPunter ||
                              (game.m_useOptions &&
                               game.m_options[siteRiver.m_river] == iPunter)))
                        {
                            continue;
                        }
                        uint32_t target = siteRiver.m_target;
                        if (!visited[target])
                        {
                            queueNext.push_back(target);
                        }
                    }
                }
                queueCurrent.clear();
            }
        }
    }
}
