#ifndef NETUTILS_HPP
#define NETUTILS_HPP

#include "Common.hpp"

bool networkStartup(std::string* pMsg = nullptr);
bool networkCleanup(std::string* pMsg = nullptr);

class Address
{
public:
    Address() = default;
    Address(const std::string& host, const std::string& port) :
        m_host(host), m_port(port) {}
    Address(const Address& other) = default;
    Address(Address&& other) = default;
    ~Address() = default;
    Address& operator=(const Address& other) = default;
    Address& operator=(Address&& other) = default;

    std::string m_host;
    std::string m_port;
};

bool lookupAddresses(const std::string& hostName,
                     const std::string& portName,
                     std::vector<Address>* pAddresses = nullptr,
                     std::string* pMsg = nullptr);

enum class SocketState : uint32_t
{
    Disconnected,
    Connecting,
    Connected,
    Error,
    ReadClosed,
    WriteClosed,
    Closed
};

class TCPSocketImpl;

class TCPSocket
{
public:
    TCPSocket();
    TCPSocket(TCPSocket&& other);
    ~TCPSocket();
    TCPSocket& operator=(TCPSocket&& other);

    SocketState getState() const;
    std::string getMsg() const;
    void connect(const Address& address);
    void update();
    void close();
    void reset();
    bool getRemoteAddress(Address* pAddress, std::string* pMsg = nullptr);
    bool getLocalAddress(Address* pAddress, std::string* pMsg = nullptr);
    uint64_t getSendStalledDurationMS() const;
    uint64_t getRecvStalledDurationMS() const;

    std::vector<uint8_t> m_sendData;
    std::vector<uint8_t> m_recvData;

private:
    friend class TCPServerSocket;

    std::unique_ptr<TCPSocketImpl> m_pImpl;

private:
    TCPSocket(const TCPSocket& other) = delete;
    TCPSocket& operator=(const TCPSocket& other) = delete;
};

class TCPServerSocketImpl;

class TCPServerSocket
{
public:
    TCPServerSocket();
    TCPServerSocket(TCPServerSocket&& other);
    ~TCPServerSocket();
    TCPServerSocket& operator=(TCPServerSocket&& other);

    bool listen(const std::string& port, std::string* pMsg = nullptr);
    bool accept(TCPSocket* pSocket, std::string* pMsg = nullptr);
    void close();

private:
    std::unique_ptr<TCPServerSocketImpl> m_pImpl;

private:
    TCPServerSocket(const TCPServerSocket& other) = delete;
    TCPServerSocket& operator=(const TCPServerSocket& other) = delete;
};

#endif
