#include "RandomSolver.hpp"
#include "Game.hpp"
#include "Map.hpp"
#include <random>

using std::vector;

RandomSolver::RandomSolver()
{
}

RandomSolver::~RandomSolver()
{
}

void RandomSolver::solve(const Game& game,
                         vector<uint32_t>* pRoute)
{
    auto& map = game.m_map;

    uint32_t optionsLeft = 0;
    if (game.m_useOptions)
    {
        optionsLeft = game.m_optionsLeft[game.m_punter];
    }

    uint32_t numRivers = map.m_numRivers;
    uint32_t numUnclaimed = 0;
    for (uint32_t iRiver = 0; iRiver < numRivers; iRiver++)
    {
        if (game.m_claims[iRiver] == (uint32_t)-1 ||
            (optionsLeft > 0 &&
             game.m_claims[iRiver] != game.m_punter &&
             game.m_options[iRiver] == (uint32_t)-1))
        {
            numUnclaimed++;
        }
    }

    std::default_random_engine gen{std::random_device()()};
    std::uniform_int_distribution<uint32_t> dist(0, numUnclaimed - 1);
    uint32_t iUnclaimed = dist(gen);

    numUnclaimed = 0;
    for (uint32_t iRiver = 0; iRiver < numRivers; iRiver++)
    {
        if (game.m_claims[iRiver] == (uint32_t)-1 ||
            (optionsLeft > 0 &&
             game.m_claims[iRiver] != game.m_punter &&
             game.m_options[iRiver] == (uint32_t)-1))
        {
            if (numUnclaimed == iUnclaimed)
            {
                auto& river = map.m_rivers[iRiver];
                pRoute->push_back(river.m_source);
                pRoute->push_back(river.m_target);
                return;
            }
            numUnclaimed++;
        }
    }

    return; // pass
}
