#include "SequentialSolver.hpp"
#include "Game.hpp"
#include "Map.hpp"

using std::vector;

SequentialSolver::SequentialSolver() :
    m_nextClaim(0)
{
}

SequentialSolver::~SequentialSolver()
{
}

void SequentialSolver::solve(const Game& game,
                             vector<uint32_t>* pRoute)
{
    auto& map = game.m_map;

    uint32_t optionsLeft = 0;
    if (game.m_useOptions)
    {
        optionsLeft = game.m_optionsLeft[game.m_punter];
    }

    uint32_t claim = (uint32_t)-1;
    if (m_nextClaim < map.m_numRivers &&
        (game.m_claims[m_nextClaim] == (uint32_t)-1 ||
         (optionsLeft > 0 &&
          game.m_claims[m_nextClaim] != game.m_punter &&
          game.m_options[m_nextClaim] == (uint32_t)-1)))
    {
        claim = m_nextClaim;
    }
    m_nextClaim++;

    if (claim != (uint32_t)-1)
    {
        auto& river = map.m_rivers[claim];
        pRoute->push_back(river.m_source);
        pRoute->push_back(river.m_target);
        return;
    }

    return; // pass
}

void SequentialSolver::serialize(vector<uint32_t>* pData) const
{
    pData->push_back(m_nextClaim);
}

bool SequentialSolver::deserialize(const vector<uint32_t>& data,
                                   size_t& pos)
{
    size_t size = data.size();
    if (pos + 1 > size)
    {
        return false;
    }

    m_nextClaim = data[pos++];
    return true;
}
