#ifndef TASKWORKER_HPP
#define TASKWORKER_HPP

#include "Common.hpp"
#include "TaskQueue.hpp"
#include "ThreadUtils.hpp"

template<class _TaskType>
class TaskWorker
{
public:
    typedef _TaskType TaskType;
    typedef decltype((*(TaskType*)nullptr)()) ResultType;

    TaskWorker(TaskQueue<TaskType>& taskQueue) :
        m_taskQueue(taskQueue) {}
    TaskWorker(const TaskWorker& other) = default;
    TaskWorker(TaskWorker&& other) = default;
    ~TaskWorker() = default;

    void operator()()
    {
        while (true)
        {
            try
            {
                auto value = m_taskQueue.pop();
                TaskType& task = value.first;
                Promise<ResultType>& promise = value.second;
                promise.setValue(task());
            }
            catch (const TaskQueueFinished&)
            {
                return;
            }
        }
    }

private:
    TaskWorker& operator=(const TaskWorker& other) = delete;
    TaskWorker& operator=(TaskWorker&& other) = delete;

    TaskQueue<TaskType>& m_taskQueue;
};

#endif
