#include "Common.hpp"
#include "FileUtils.hpp"
#include "Protocol.hpp"
#include "Convert.hpp"
#include "MapGraphics.hpp"

using std::string;
using std::vector;

void usage()
{
    printf("Usage: showmap <mapfile>\n");
}

int main(int argc, char *argv[])
{
    if (argc != 2)
    {
        usage();
        return 1;
    }

    string fileName(argv[1]);

    FILE* f = fopen(fileName.c_str(), "rb");
    if (!f)
    {
        printf("Error opening map file\n");
        return 1;
    }

    std::vector<uint8_t> data;
    readFile(f, &data);
    fclose(f);
    const char* rawData = (const char*)&data[0];
    string strData(rawData, data.size());

    string msg;

    Protocol::Map protocolMap;
    if (!Protocol::parseMap(strData, &protocolMap, &msg))
    {
        printf("%s\n", msg.c_str());
        return 1;
    }

    Map map;
    if (!convertMap(protocolMap, &map, &msg))
    {
        printf("%s\n", msg.c_str());
        return 1;
    }

    MapGraphics mapGraphics(fileName,
                            map);

    mapGraphics.show();

    return 0;
}
