#include "Common.hpp"
#include "Program.hpp"
#include "Solver.hpp"
#include "SolverFactory.hpp"

using std::string;
using std::vector;

string getSolverUsage()
{
    string str;
    str += "\n";
    str += "Solvers:\n";

    vector<string> nameList(SolverFactory::getList());
    for (auto& name : nameList)
    {
        str += "  ";
        str += name;
        str += "\n";
    }

    return str;
}

void usage()
{
    fprintf(stderr, "Usage: solve <solver> [[<host>:]<port>]\n");
    fprintf(stderr, "%s", getSolverUsage().c_str());
}

int main(int argc, char *argv[])
{
    if (argc != 2 && argc != 3)
    {
        usage();
        return 1;
    }

    string solverName(argv[1]);
    vector<string> args;
    for (int iArg = 2; iArg < argc; iArg++)
    {
        args.push_back(argv[iArg]);
    }

    std::unique_ptr<Solver> pSolver(SolverFactory::create(solverName));
    if (!pSolver)
    {
        usage();
        return 1;
    }

    Program program(*pSolver, solverName, "solve <solver>", getSolverUsage());
    return program.run(args);
}
